// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

#include "types.h"
#include "errors.h"
#include "basic.h"
#include "graphics.h"
#include "document.h"
#include "media.h"
#include "misc.h"

namespace SX
{
	interface IAppSettings;
	interface IAppFactory;

	//////////////////////////////////////////////////////////////////////////
	// main library interface
	interface IApplication
	{
		//////////////////////////////////////////////////////////////////////////
		// properties
		virtual sx_str get_Version() const = 0;
		virtual sx_bool get_Authorized() const = 0;
		virtual IAppSettings* get_Settings() const = 0;
		virtual IAppFactory* get_Factory() const = 0;

		//////////////////////////////////////////////////////////////////////////
		// methods
		virtual void Authorize(sx_str sLicenseKey, sx_str sLicenseFilePath = nullptr) = 0; // can throw IException
	};

	//////////////////////////////////////////////////////////////////////////
	// application settings
	interface IAppSettings
	{
		//////////////////////////////////////////////////////////////////////////
		// properties
		virtual void set_DeploymentFolder(sx_str sFolder) = 0;
		virtual sx_str get_DeploymentFolder() const = 0;
		virtual sx_uint get_AggressiveMemoryUsage() = 0; // 0..1024
		virtual void set_AggressiveMemoryUsage(sx_uint uLevel) = 0;
		virtual sx_bool get_MemoryFileCaching() = 0; // use temp files for caches
		virtual void set_MemoryFileCaching(sx_bool bEnable) = 0;
		virtual sx_bool get_UseColorProfiles() = 0; 
		virtual void set_UseColorProfiles(sx_bool bEnable) = 0;
		virtual Graphics::TextRender get_PreferredTextRender () const = 0;
		virtual void set_PreferredTextRender(Graphics::TextRender v) = 0;
		virtual Graphics::ImageInterpolation get_PreferredImageInterpolation () const = 0;
		virtual void set_PreferredImageInterpolation(Graphics::ImageInterpolation v) = 0;
		virtual const sx_color_i_rgb &get_PreferredUniformTextColor() const = 0;
		virtual void set_PreferredUniformTextColor(const sx_color_i_rgb& c) = 0;
		virtual sx_int get_LocalTimeOffset() = 0;
		virtual void set_LocalTimeOffset(sx_int iMinutes) = 0;
		virtual sx_str get_UserName() const = 0;
		virtual void set_UserName(sx_str sName) = 0;
		virtual LangCode2 get_Language() const = 0;
		virtual void set_Language(LangCode2 lc) = 0;
		virtual ITrustedIdentities* get_TrustedIdentities() const = 0;
		//////////////////////////////////////////////////////////////////////////
		// methods
		virtual void ScanSystemFonts(sx_bool bSkip = false) = 0;
		virtual sx_str GetSystemFont(sx_str pPSName, sx_uint& weight, sx_uint& style, sx_uint& stretch) = 0;
		virtual void SetSystemFontSubst(sx_str pUnkFontFamily, sx_uint unk_weight, sx_uint unk_style, sx_uint unk_stretch, sx_str FontFamily, sx_uint weight = 400, sx_uint style = 0, sx_uint stretch = 5 ) = 0;
		virtual void SetSystemFontSubst(sx_str pPSName, sx_str FontFamily, sx_uint weight = 400, sx_uint style = 0, sx_uint stretch = 5 ) = 0;
		virtual void SetCustomFontSubst(sx_str pPSName, ISequentialStream* pStream ) = 0;
	};

	namespace PDF { interface IDocument; } 
	namespace XPS { interface IPackage; }
	namespace OOXML { interface IPackage; }
	namespace Viewer { interface IViewer; }
	namespace Media { interface IMedia; }
	namespace XHTML { interface ITextBox; interface IRichTextDocument;}
	namespace NAV { interface INavPackage; }
	namespace Layout { interface IDocument; }
	namespace XMP { interface IDocument; }
	namespace Barcode { interface IBarcode; enum barcode_format; }
	namespace ZUGFeRD { interface IInvoiceStream; interface IInvoiceDocument; }
	namespace EInvoice { interface IInvoiceDocument; }
	namespace OCF { interface IPackage; }
	namespace FB { interface IPackage; }
	namespace PALM { interface IPackage; }
	namespace ZIP { interface IPackage; }
#ifdef SX_WINDESKTOP_DLL
	namespace Viewer { interface IWinViewer; typedef HWND sx_window; }
#endif

	//////////////////////////////////////////////////////////////////////////
	// objects factory
	interface IAppFactory
	{
		// streams
		virtual ISequentialStream* CreateSequentialStream() = 0;
		virtual ISequentialStream* CreateReadStream(sx_str sFilePath) = 0;
		virtual ISequentialStream* CreateWriteStream(sx_str sFilePath) = 0;
		virtual ISequentialStream* CreateRefStream(sx_com_stream pIStream) = 0;
		// crypto
		virtual ICryptoCert* CreateCryptoCert() = 0;
		virtual ICryptoKey* CreateCryptoKey() = 0;
		// graphics
		virtual Graphics::IRegion* CreateRegion() = 0;
		virtual Graphics::IColor* Color() = 0;
		virtual Graphics::IMatrix* Matrix() = 0;
		virtual Graphics::IBitmapData* CreateBitmapFromStream(ISequentialStream* pStream) = 0;
		virtual Graphics::IBitmapData* CreateBitmapFromSource(sx_wic_bitmap_source pSource) = 0;
		virtual Graphics::IBitmapData* CreateBitmapFromBits(sx_uint width, sx_uint height, Graphics::PixelFormat format, const sx_byte* pBits) = 0;
		virtual Graphics::IInkImage* CreateInkImage(ISequentialStream* pStream) = 0;
		virtual Graphics::IDrawingContext* CreateDrawingContext(SX::Graphics::Drawer Dr) = 0;
		// documents
		virtual ITextDocument* CreateTextDocument() = 0;
		virtual XHTML::IRichTextDocument* CreateRichTextDocument() = 0;
		virtual IImageDocument* CreateImageDocument() = 0;
		virtual PDF::IDocument* CreatePDFDocument() = 0;
		virtual XPS::IPackage* CreateXPSPackage() = 0;
		virtual OOXML::IPackage* CreateOOXMLPackage() = 0;
		virtual OCF::IPackage* CreateOCFPackage() = 0;
		virtual PALM::IPackage* CreatePALMPackage() = 0;
		virtual FB::IPackage* CreateFBPackage() = 0;
		virtual ZIP::IPackage* CreateZIPPackage() = 0;
		virtual IFragmentDocument* CreateFragmentDocument() = 0;
		virtual ICompositeDocument* CreateCompositeDocument() = 0;
		virtual IAcquisitionDocument* CreateAcquisitionDocument() = 0;
		virtual Layout::IDocument* CreateLayoutDocument() = 0;
		virtual ZUGFeRD::IInvoiceDocument* CreateInvoiceDocument() = 0;
		virtual EInvoice::IInvoiceDocument* CreateEInvoiceDocument() = 0;

		// miscellaneous
		virtual Media::IMedia* Media() = 0;
		virtual XHTML::ITextBox* CreateTextBox() = 0;
		virtual NAV::INavPackage* CreateNavPackage() = 0;
		virtual XMP::IDocument* CreateXMPDocument() = 0;
		virtual Barcode::IBarcode* CreateBarcode(SX::Barcode::barcode_format format) = 0;
		virtual ZUGFeRD::IInvoiceStream* CreateInvoiceStream() = 0;

		// viewer
		virtual Viewer::IViewer* CreateViewer() = 0;
#ifdef SX_WINDESKTOP_DLL
		virtual Viewer::IWinViewer* CreateWinViewer(sx_rect& rc, Viewer::sx_window parent, sx_flags style) = 0;
#endif
	};

	// library entry point
	typedef IApplication* (__cdecl * LibraryEntryPoint)();
#ifdef SX_UNIX_DLL
	#if defined (__cplusplus)
		#define __cdecl
		extern "C" { IApplication* EntryPoint(); }
	#endif
#endif
} // namespace SX
