// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

#ifndef __WINCRYPT_H__
	struct CERT_CONTEXT; // Windows Crypto API
#endif
interface IStream; 
typedef IStream* sx_com_stream;

namespace SX
{
	//////////////////////////////////////////////////////////////////////////
	// base object, responsible for managing lifetime using a standard reference counting scheme
	typedef const sx_uint object_type;
	const object_type 
		obj_type_base		= 0x01,
		obj_type_string		= 0x02,
		obj_type_stream		= 0x03,
		obj_type_crypto_key	= 0x04,
		obj_type_crypto_cert= 0x05,
		// free for use 
		obj_type_next_value = 0x39;

	interface IRefObject
	{
		virtual object_type get_object_type() const = 0;
		virtual sx_uint AddRef() = 0;
		virtual sx_uint Release() = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	// unicode string
	interface IStr : public IRefObject
	{
		virtual sx_str get_str() = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	// data stream with sequential access
	interface ISequentialStream : public IRefObject
	{
		virtual sx_qword GetLength() = 0;
		virtual void     Reset() = 0;
		virtual sx_uint  Read(sx_byte* Buf, sx_uint Size) = 0;
		virtual sx_uint  Write(const sx_byte* Buf, sx_uint Size) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	// crypto key
	interface ICryptoKey : public IRefObject
	{
		virtual void OpenFromPEM(ISequentialStream* pKey) = 0;
		virtual void OpenFromPVK(ISequentialStream* pKey, sx_str sPassword) = 0;
	};

	typedef const CERT_CONTEXT* sx_cert_context;

	enum sx_cert_ku 
	{
		sx_cert_ku_dig_sign	= 0x001,
		sx_cert_ku_non_rep	= 0x002,
		sx_cert_ku_key_enc	= 0x004,
		sx_cert_ku_data_enc	= 0x008,
		sx_cert_ku_key_agr	= 0x010,
		sx_cert_ku_cert_sign= 0x020,
		sx_cert_ku_crl_sign	= 0x040,
		sx_cert_ku_enc_only	= 0x080,
		sx_cert_ku_dec_only	= 0x100
	};

	//////////////////////////////////////////////////////////////////////////
	// X509 or PKCS7 encoded certificate
	interface ICryptoCert : public IRefObject
	{
		virtual IStr* get_Subject() const = 0;
		virtual IStr* get_Issuer() const = 0;
		virtual sx_time get_ValidFrom() = 0;
		virtual sx_time get_ValidTo() = 0;

		virtual void Create(const sx_str TechSubject, sx_time From, sx_time To, sx_flags KU, sx_bool bCA = false, ICryptoCert *pCert = nullptr) = 0;
		virtual void OpenFromCER(ISequentialStream* pCert) = 0;
		virtual void OpenFromSPC(ISequentialStream* pCert) = 0;
		virtual void OpenFromPFX(ISequentialStream* pCert, sx_str sPassword) = 0;
		virtual void SaveToPFX(ISequentialStream* pCert, sx_str sPassword) = 0;
		virtual ICryptoCert* GetIssuerCert() = 0;
		virtual ICryptoKey* AcquirePrivateKey() = 0;
		// Gate to Windows CryptoAPI
		virtual void LoadFromCryptoContext(sx_cert_context pContext) = 0;
		virtual sx_cert_context CreateCryptoContext() const = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	// identtities
	interface ITrustedIdentities
	{
		//////////////////////////////////////////////////////////////////////////
		// properties
		virtual sx_uint get_Count() = 0;

		//////////////////////////////////////////////////////////////////////////
		// methods
		virtual ICryptoCert* Item(sx_uint uIndex) = 0;
		virtual void Add(ICryptoCert* pCert) = 0;
		virtual void Remove(ICryptoCert* pCert) = 0;
		virtual sx_bool IsTrusted(ICryptoCert* pCert) = 0;

		virtual void Load(ISequentialStream* pStore) = 0;
		virtual void Save(ISequentialStream* pStore) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ILoadStatus
	{
		virtual sx_bool OnPage(sx_uint ind) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ILoadProvider
	{
		virtual ISequentialStream* GetResource(sx_str sURL, sx_str sContType) = 0;
		virtual ILoadProvider* GetResourceProvider(sx_str sURL) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IPasswordVerifier : public IRefObject
	{
		virtual sx_bool VerifyPassword(sx_str sPassword, sx_uint res) const = 0;
	};


	//////////////////////////////////////////////////////////////////////////
	namespace ZIP
	{
		//////////////////////////////////////////////////////////////////////////
		interface IStream
		{
			virtual sx_str get_Name() const = 0;
			virtual ISequentialStream* get_Data() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IFolder
		{
			virtual sx_str get_Name() const = 0;
			virtual IFolder* GetSubFolder(IFolder* pPrev = nullptr) const = 0;
			virtual IStream* GetStream(IStream* pPrev = nullptr) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPackage : public IRefObject
		{
			virtual IFolder* get_Folder() const = 0;
			virtual void Open(ISequentialStream* pStream) = 0;
		};
	} // namespace ZIP
	
#ifdef SX_USE_OBJ_PTR
	template<typename _T> class ObjPtr 
	{
		public:

		ObjPtr() : m_p(nullptr) { }

		ObjPtr(_T* p) : m_p(p) { }

		ObjPtr(const ObjPtr<_T>& ptr) : m_p(ptr.m_p) { if(m_p) m_p->AddRef(); }

		~ObjPtr() { if(m_p) m_p->Release(); }

		inline void reset(_T* p = nullptr) { if(m_p != p) { if(m_p) m_p->Release(); m_p = p; } }

		inline _T* release() { _T* p(m_p); m_p = nullptr; return p; }

		inline _T* get() const { return m_p; }

		inline operator _T* () const { return m_p; }

		inline _T* operator -> () const { return m_p; }

		inline const ObjPtr<_T>& operator = (_T* p) { reset(p); return *this; }

		inline const ObjPtr<_T> operator = (const ObjPtr<_T>& ptr) { reset(ptr.m_p); if(m_p) m_p->AddRef(); return *this; }

		protected: _T* m_p;
	};
#endif
}