﻿using SX.UWP;
using SX.UWP.Viewer;
using SX.UWP.PDF;
using SX.UWP.Graphics;
using System;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Core;
using Windows.UI.Popups;
using System.Threading;
using Windows.Graphics.Printing;

namespace Sample
{
    internal enum FileType
    {
        PDF = 1,
        XPS,
        Image,
        Text
    }

    public class ViewerPageContext : BasicDataContext
    {
        private IBaseDocument m_doc;
        private bool m_IsProgressActive;
        private bool m_IsPaneOpened;
        private WeakReference m_control;

        public ViewerPageContext(PDFXpansionControl control)
        {
            if(Tools.AuthorizeSDK())
            {
                m_control = new WeakReference(control);
                control.Events.OnAttachDoc += AttachDocHandler;
                control.Events.OnChangedLayout += ChangedLayoutHandler;

                control.Config.PDF.Popups = viewer_popups.viewer_popups_multi;
                control.Config.PageCacheSize = 50;
            }
        }

        public bool IsProgressActive
        {
            get { return m_IsProgressActive;  }
            private set { m_IsProgressActive = value; RaisePropertyChangedEventAsync("IsProgressActive"); }
        }

        public bool IsPaneOpened
        {
            get { return m_IsPaneOpened; }
            set { m_IsPaneOpened = value; RaisePropertyChangedEventAsync("IsPaneOpened"); }
        }

        public string Title { get; private set; }

        public bool IsDocumentValid
        {
            get { return m_doc != null; }
        }

        public ViewableDocument ViewerDocument
        {
            get { return m_doc != null ? m_doc.GetViewableDocument() : null;  }
        }

        public uint CanvasMode
        {
            get { return (uint)GetControl().Config.Mode; }
            set
            {
                if (value != uint.MaxValue)
                {
                    GetControl().Config.Mode = (viewer_mode)value;
                    GetControl().Relayout(GetControl().Canvas.ActivePage, true);
                    RaisePropertyChangedEventAsync("CanvasMode");
                }
            }
        }

        public bool VerticalLayout
        {
            get { return GetControl().Canvas.VerticalLayout; }
            set { GetControl().Canvas.VerticalLayout = value; }
        }

        public bool CoverPage
        {
            get { return GetControl().Canvas.SeparateCover; }
            set { GetControl().Canvas.SeparateCover = value; }
        }

        public uint LayoutMultiplicity
        {
            get { return GetControl().Canvas.LayoutMultiplicity; }
            set
            {
                if(value != uint.MaxValue)
                {
                    GetControl().Canvas.LayoutMultiplicity = value;
                    RaisePropertyChangedEventAsync("LayoutMultiplicity");
                }
            }
        }

        public uint Zoom
        {
            get { return (uint)(GetControl().Canvas.Zoom * 100); }
            set
            {
                if(value != uint.MaxValue)
                {
                    GetControl().Canvas.Zoom = value / 100.0;
                    RaisePropertyChangedEventAsync("Zoom");
                }
            }
        }

        public uint FitMode
        {
            get { return (uint)GetControl().Canvas.Fit; }
            set
            {
                if (value != uint.MaxValue)
                {
                    GetControl().Canvas.Fit = (viewer_fit)value;
                    RaisePropertyChangedEventAsync("FitMode");
                }
            }
        }

        public ICommand CmdMain
        {
            get { return new RelayCommand((object args) => { IsPaneOpened = !IsPaneOpened; }); }
        }

        public ICommand CmdOpen
        {
            get { return new RelayCommand((object args) => { OpenFile((FileType)Convert.ToUInt32(args)); }); }
        }

        public ICommand CmdRotate
        {
            get
            {
                return new RelayCommand((object args) =>
                {
                    int angle = Convert.ToInt32(args);
                    var canvas = GetControl().Canvas;
                    if (angle > 0)
                    {
                        int rotation = (int)canvas.Rotation;
                        canvas.Rotation = (sx_rotation_angle)((rotation + 90) % 360);
                    }
                    else
                    {
                        int rotation = (int)canvas.Rotation;
                        rotation -= 90;
                        canvas.Rotation = rotation < 0 ? sx_rotation_angle.sx_rotation_angle_270 : (sx_rotation_angle)rotation;
                    }
                });
            }
        }

        public ICommand CmdPrint
        {
            get
            {
                return new RelayCommand((object args) => 
                {                    
                    PrintManager.GetForCurrentView().PrintTaskRequested += OnPrintTaskRequested;
                    PrintManager.ShowPrintUIAsync();
                });
            }
        }

        public bool IsProgressTextVisible { get; private set; }
        public string ProgressText { get; private set; }

        private async Task<StorageFile> SelectFileAsync(FileType type)
        {
            var openPicker = new FileOpenPicker();
            openPicker.ViewMode = PickerViewMode.List;
            openPicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;

            switch(type)
            {
                case FileType.PDF:
                    openPicker.FileTypeFilter.Add(".pdf");
                    break;
                case FileType.XPS:
                    openPicker.FileTypeFilter.Add(".xps");
                    openPicker.FileTypeFilter.Add(".oxps");
                    break;
                case FileType.Image:
                    openPicker.FileTypeFilter.Add(".jpg");
                    openPicker.FileTypeFilter.Add(".jpeg");
                    openPicker.FileTypeFilter.Add(".png");
                    openPicker.FileTypeFilter.Add(".tiff");
                    openPicker.FileTypeFilter.Add(".gif");
                    openPicker.FileTypeFilter.Add(".bmp");
                    break;
                case FileType.Text:
                    openPicker.FileTypeFilter.Add(".txt");
                    break;
            }
            return await openPicker.PickSingleFileAsync();
        }

        private async void OpenFile(FileType type)
        {
            IsPaneOpened = false;
            var file = await SelectFileAsync(type);
            if (file != null)
            {
                IsProgressActive = true;
                try
                {
                    await Task.Run(() =>
                    {
                        var stream = PDFXpansionSDK.EntryPoint().Factory.CreateReadStreamFromFile(file);
                        switch (type)
                        {
                            case FileType.PDF: OpenPDF(stream); break;
                            case FileType.XPS: OpenXPS(stream); break;
                            case FileType.Image: OpenImage(stream); break;
                            case FileType.Text: OpenText(stream); break;
                        }
                    });

                    Title = file.Name;
                    UpdateViewer();
                }
                catch (Exception)
                {
                    IsProgressActive = false;
                    await (new MessageDialog("The error occurred during opening file.")).ShowAsync();
                }
            }
        }
        
        private void OpenPDF(SequentialStream stream)
        {
            var doc = PDFXpansionSDK.EntryPoint().Factory.CreatePDFDocument();
            doc.OpenStream(stream);
            switch (doc.Properties.Security.Security)
            {
                case doc_security.doc_security_none:
                    break;
                case doc_security.doc_security_password:
                    {
                        var ps = doc.Properties.Security.StandardSecurity() as PasswordSecurity;
                        if (!ps.SetUserPassword(""))
                        {
                            var completed = new ManualResetEvent(false);
                            Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async () =>
                            {
                                var passDlg = new PasswordDialog(new ValidatePasswordHandler((password) => { return ps.SetUserPassword(password); }));
                                await passDlg.ShowAsync();
                                completed.Set();
                            }).AsTask().Wait();

                            completed.WaitOne();
                            if(!ps.IsPasswordSet(true)) throw new NotImplementedException();
                        }
                    }
                    break;
                default:
                    throw new NotImplementedException();
            }
            m_doc = doc;
        }

        private void OpenXPS(SequentialStream stream)
        {
            var pack = PDFXpansionSDK.EntryPoint().Factory.CreateXPSPackage();
            pack.OpenStream(stream);
            m_doc = pack;
        }

        private void OpenImage(SequentialStream stream)
        {
            var factory = PDFXpansionSDK.EntryPoint().Factory;
            var img = factory.CreateImageDocument();
            img.SetImage(factory.CreateBitmapFromStream(stream), 96);
            m_doc = img;
        }

        private void OpenText(SequentialStream stream)
        {
            var txt = PDFXpansionSDK.EntryPoint().Factory.CreateTextDocument();
            txt.SetText(stream, new page_props() { width=500, height=800, bottom=10, left=10, right=10, top=10 }, false, 0);
            m_doc = txt;
        }

        private void UpdateViewer()
        {
            RaisePropertyChangedEventAsync("Title");
            RaisePropertyChangedEventAsync("IsDocumentValid");
            RaisePropertyChangedEventAsync("ViewerDocument");
        }

        private PDFXpansionControl GetControl()
        {
            return m_control.Target as PDFXpansionControl;
        }

        private void AttachDocHandler(IViewerEvent_Doc ve)
        {
            IsProgressActive = false;
        }

        private void ChangedLayoutHandler(IViewerEvent ve)
        {
            RaisePropertyChangedEventAsync("Zoom");
            RaisePropertyChangedEventAsync("FitMode");
        }

        private void OnPrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs e)
        {
            PrintTask printTask = null;
            var printProv = m_doc.GetPrintProvider();
            printProv.DrawOptions = draw_opt.draw_opt_allow_smooth_lines | draw_opt.draw_opt_rasterize_blended;
            printProv.PDFAnnots = annot_type.annot_all;

            printTask = e.Request.CreatePrintTask("PrintJob Title", sourceRequested =>
            {
                printTask.Completed += (s, args) =>
                {
                    Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                    {
                        PrintManager.GetForCurrentView().PrintTaskRequested -= OnPrintTaskRequested;
                        GetControl().Suspended = false;
                        GetControl().Invalidate(false, true);
                        IsProgressActive = false;
                        UpdatePrintProgress(uint.MaxValue);
                    });
                };

                printTask.Progressing += (s, args) => 
                {
                    Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => {
                        UpdatePrintProgress(args.DocumentPageCount);
                    });
                };

                sourceRequested.SetSource(printProv.GetPrintDocumentSource());
                IsProgressActive = true;
                GetControl().Suspended = true;
            });
        }

        private void UpdatePrintProgress(uint page)
        {
            var showProgress = page != uint.MaxValue;
            if(showProgress != IsProgressTextVisible)
            {
                IsProgressTextVisible = showProgress;
                RaisePropertyChangedEventAsync("IsProgressTextVisible");
            }
            if(showProgress)
            {
                ProgressText = "Pages printed: " + page.ToString();
                RaisePropertyChangedEventAsync("ProgressText");
            }
        }

    }
}