﻿using SX.UWP;
using SX.UWP.Viewer;
using SX.UWP.PDF;
using System;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Core;
using Windows.UI.Popups;
using System.Threading;
using System.Collections.Generic;

namespace Sample
{
    internal enum FileType
    {
        PDF = 1,
        XPS,
        Image,
        Text
    }

    public class ViewerPageContext : BasicDataContext
    {
        private IBaseDocument m_doc;
        private bool m_IsProgressActive;
        private bool m_IsPaneOpened;
        private WeakReference m_control;

        private FileType m_type;
        private IToolState m_tool;
        private uint m_ToolActivePage;

        public ViewerPageContext(PDFXpansionControl control)
        {
            if(Tools.AuthorizeSDK())
            {
                m_control = new WeakReference(control);
                control.Events.OnAttachDoc += AttachDocHandler;
                control.Events.OnChangedLayout += ChangedLayoutHandler;
                control.Events.SplitTools = true;
                control.Events.OnToolInk += ToolInkHandler;
                control.Events.OnToolEraser += ToolEraserHandler;

                control.Config.PDF.Popups = viewer_popups.viewer_popups_multi;
                control.Config.PageCacheSize = 50;
            }
        }

        public bool IsProgressActive
        {
            get { return m_IsProgressActive;  }
            private set { m_IsProgressActive = value; RaisePropertyChangedEventAsync("IsProgressActive"); }
        }

        public bool IsPaneOpened
        {
            get { return m_IsPaneOpened; }
            set { m_IsPaneOpened = value; RaisePropertyChangedEventAsync("IsPaneOpened"); }
        }

        public string Title { get; private set; }

        public bool IsDocumentValid
        {
            get { return m_doc != null; }
        }

        public ViewableDocument ViewerDocument
        {
            get { return m_doc != null ? m_doc.GetViewableDocument() : null;  }
        }

        public uint CanvasMode
        {
            get { return (uint)GetControl().Config.Mode; }
            set
            {
                if (value != uint.MaxValue)
                {
                    GetControl().Config.Mode = (viewer_mode)value;
                    GetControl().Relayout(GetControl().Canvas.ActivePage, true);
                    RaisePropertyChangedEventAsync("CanvasMode");
                }
            }
        }

        public bool VerticalLayout
        {
            get { return GetControl().Canvas.VerticalLayout; }
            set { GetControl().Canvas.VerticalLayout = value; }
        }

        public bool CoverPage
        {
            get { return GetControl().Canvas.SeparateCover; }
            set { GetControl().Canvas.SeparateCover = value; }
        }

        public uint LayoutMultiplicity
        {
            get { return GetControl().Canvas.LayoutMultiplicity; }
            set
            {
                if(value != uint.MaxValue)
                {
                    GetControl().Canvas.LayoutMultiplicity = value;
                    RaisePropertyChangedEventAsync("LayoutMultiplicity");
                }
            }
        }

        public uint Zoom
        {
            get { return (uint)(GetControl().Canvas.Zoom * 100); }
            set
            {
                if(value != uint.MaxValue)
                {
                    GetControl().Canvas.Zoom = value / 100.0;
                    RaisePropertyChangedEventAsync("Zoom");
                }
            }
        }

        public uint FitMode
        {
            get { return (uint)GetControl().Canvas.Fit; }
            set
            {
                if (value != uint.MaxValue)
                {
                    GetControl().Canvas.Fit = (viewer_fit)value;
                    RaisePropertyChangedEventAsync("FitMode");
                }
            }
        }

        public ICommand CmdMain
        {
            get { return new RelayCommand((object args) => { IsPaneOpened = !IsPaneOpened; }); }
        }

        public ICommand CmdOpen
        {
            get { return new RelayCommand((object args) => { OpenFile((FileType)Convert.ToUInt32(args)); }); }
        }

        public ICommand CmdRotate
        {
            get
            {
                return new RelayCommand((object args) =>
                {
                    int angle = Convert.ToInt32(args);
                    var canvas = GetControl().Canvas;
                    if (angle > 0)
                    {
                        int rotation = (int)canvas.Rotation;
                        canvas.Rotation = (sx_rotation_angle)((rotation + 90) % 360);
                    }
                    else
                    {
                        int rotation = (int)canvas.Rotation;
                        rotation -= 90;
                        canvas.Rotation = rotation < 0 ? sx_rotation_angle.sx_rotation_angle_270 : (sx_rotation_angle)rotation;
                    }
                });
            }
        }

        public ICommand CmdSave
        {
            get { return new RelayCommand(async (object args) => 
            {
                var savePicker = new FileSavePicker();
                savePicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;
                savePicker.FileTypeChoices.Add("PDF Files", new List<string>() { ".pdf" });
                savePicker.SuggestedFileName = "unknown";

                var file = await savePicker.PickSaveFileAsync();
                if(file != null)
                {
                    var stream = PDFXpansionSDK.EntryPoint().Factory.CreateSequentialStream();
                    (m_doc as Document).SaveAsStream(stream);
                    await stream.WriteFileAsync(file);
                }

            }); }
        }

        public ICommand CmdInk
        {
            get
            {
                return new RelayCommand((object args) =>
                {
                    GetControl().ToolMode = tool_mode.tool_mode_ink;
                    RaisePropertyChangedEventAsync("IsToolActive");
                });
            }
        }

        public ICommand CmdEraser
        {
            get
            {
                return new RelayCommand((object args) =>
                {
                    GetControl().ToolMode = tool_mode.tool_mode_eraser;
                    RaisePropertyChangedEventAsync("IsToolActive");
                });
            }
        }

        public ICommand CmdCloseTool
        {
            get
            {
                return new RelayCommand((object args) =>
                {
                    GetControl().ToolMode = tool_mode.tool_mode_none;
                    m_tool = null;
                    RaisePropertyChangedEventAsync("IsToolActive");
                });
            }
        }

        public bool IsSaveAvailable
        {
            get { return m_type == FileType.PDF ? (m_doc as Document).Properties.Modified : false; }
        }

        public bool IsToolsAvailable
        {
            get { return m_type == FileType.PDF; }
        }

        public bool IsToolActive
        {
            get { return m_tool != null; }
        }

        private async Task<StorageFile> SelectFileAsync(FileType type)
        {
            var openPicker = new FileOpenPicker();
            openPicker.ViewMode = PickerViewMode.List;
            openPicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;

            switch(type)
            {
                case FileType.PDF:
                    openPicker.FileTypeFilter.Add(".pdf");
                    break;
                case FileType.XPS:
                    openPicker.FileTypeFilter.Add(".xps");
                    openPicker.FileTypeFilter.Add(".oxps");
                    break;
                case FileType.Image:
                    openPicker.FileTypeFilter.Add(".jpg");
                    openPicker.FileTypeFilter.Add(".jpeg");
                    openPicker.FileTypeFilter.Add(".png");
                    openPicker.FileTypeFilter.Add(".tiff");
                    openPicker.FileTypeFilter.Add(".gif");
                    openPicker.FileTypeFilter.Add(".bmp");
                    break;
                case FileType.Text:
                    openPicker.FileTypeFilter.Add(".txt");
                    break;
            }
            return await openPicker.PickSingleFileAsync();
        }

        private async void OpenFile(FileType type)
        {
            IsPaneOpened = false;
            var file = await SelectFileAsync(type);
            if (file != null)
            {
                IsProgressActive = true;
                try
                {
                    await Task.Run(() =>
                    {
                        var stream = PDFXpansionSDK.EntryPoint().Factory.CreateReadStreamFromFile(file);
                        switch (type)
                        {
                            case FileType.PDF: OpenPDF(stream); break;
                            case FileType.XPS: OpenXPS(stream); break;
                            case FileType.Image: OpenImage(stream); break;
                            case FileType.Text: OpenText(stream); break;
                        }
                    });

                    Title = file.Name;
                    UpdateViewer();
                    m_type = type;
                    m_tool = null;
                    RaisePropertyChangedEventAsync("IsToolActive");
                    RaisePropertyChangedEventAsync("IsSaveAvailable");
					RaisePropertyChangedEventAsync("IsToolsAvailable");
                }
                catch (Exception)
                {
                    IsProgressActive = false;
                    await (new MessageDialog("The error occurred during opening file.")).ShowAsync();
                }
            }
        }
        
        private void OpenPDF(SequentialStream stream)
        {
            var doc = PDFXpansionSDK.EntryPoint().Factory.CreatePDFDocument();
            doc.OpenStream(stream);
            switch (doc.Properties.Security.Security)
            {
                case doc_security.doc_security_none:
                    break;
                case doc_security.doc_security_password:
                    {
                        var ps = doc.Properties.Security.StandardSecurity() as PasswordSecurity;
                        if (!ps.SetUserPassword(""))
                        {
                            var completed = new ManualResetEvent(false);
                            Dispatcher.RunAsync(CoreDispatcherPriority.Normal, async () =>
                            {
                                var passDlg = new PasswordDialog(new ValidatePasswordHandler((password) => { return ps.SetUserPassword(password); }));
                                await passDlg.ShowAsync();
                                completed.Set();
                            }).AsTask().Wait();

                            completed.WaitOne();
                            if(!ps.IsPasswordSet(true)) throw new NotImplementedException();
                        }
                    }
                    break;
                default:
                    throw new NotImplementedException();
            }
            m_doc = doc;
        }

        private void OpenXPS(SequentialStream stream)
        {
            var pack = PDFXpansionSDK.EntryPoint().Factory.CreateXPSPackage();
            pack.OpenStream(stream);
            m_doc = pack;
        }

        private void OpenImage(SequentialStream stream)
        {
            var factory = PDFXpansionSDK.EntryPoint().Factory;
            var img = factory.CreateImageDocument();
            img.SetImage(factory.CreateBitmapFromStream(stream), 96);
            m_doc = img;
        }

        private void OpenText(SequentialStream stream)
        {
            var txt = PDFXpansionSDK.EntryPoint().Factory.CreateTextDocument();
            txt.SetText(stream, new page_props() { width=500, height=800, bottom=10, left=10, right=10, top=10 }, false, 0);
            m_doc = txt;
        }

        private void UpdateViewer()
        {
            RaisePropertyChangedEventAsync("Title");
            RaisePropertyChangedEventAsync("IsDocumentValid");
            RaisePropertyChangedEventAsync("ViewerDocument");
        }

        private PDFXpansionControl GetControl()
        {
            return m_control.Target as PDFXpansionControl;
        }

        private Ink CreateNewInkAnnot(uint pageIndex)
        {
            var page = (m_doc as Document).Pages.Item(pageIndex);
            var newink = page.Annots.InsertNew(annot_type.annot_type_ink, null) as Ink;
            newink.Color = new sx_color_i_rgb(255, 0, 0);
            newink.BorderWidth = 10;
            return newink;
        }

        private void AddInksToEraser(ToolEraser tool)
        {
            var annots = (m_doc as Document).Pages.Item(tool.Page).Annots;
            for(uint i=0; i<annots.Count; ++i)
            {
                var annot = annots.Item(i);
                if (annot.Type == annot_type.annot_type_ink)
                    tool.AddInk(annot as Ink);
            }
        }

        private void AttachDocHandler(IViewerEvent_Doc ve)
        {
            IsProgressActive = false;
        }

        private void ChangedLayoutHandler(IViewerEvent ve)
        {
            RaisePropertyChangedEventAsync("Zoom");
            RaisePropertyChangedEventAsync("FitMode");
        }

        private void ToolInkHandler(ViewerEvent_ToolActivity ta, ToolInk t)
        {
            m_tool = ta.State;
            switch (m_tool.Activity)
            {
                case sx_tool_act.sx_tool_change:
                    m_ToolActivePage = t.Page = GetControl().Canvas.ActivePage;
                    t.Ink = CreateNewInkAnnot(t.Page);
                    break;

                case sx_tool_act.sx_tool_start:
                    if (t.Page != m_ToolActivePage)
                    {
                        t.Ink = CreateNewInkAnnot(t.Page);
                        m_ToolActivePage = t.Page;
                    }
                    break;

                case sx_tool_act.sx_tool_apply:
                    RaisePropertyChangedEventAsync("IsSaveAvailable");
                    break;
            }
        }

        private void ToolEraserHandler(ViewerEvent_ToolActivity ta, ToolEraser t)
        {
            m_tool = ta.State;
            switch (m_tool.Activity)
            {
                case sx_tool_act.sx_tool_change:
                    m_ToolActivePage = t.Page = GetControl().Canvas.ActivePage;
                    AddInksToEraser(t);
                    t.Size = 30;
                    break;

                case sx_tool_act.sx_tool_start:
                    if (t.Page != m_ToolActivePage)
                    {
                        AddInksToEraser(t);
                        m_ToolActivePage = t.Page;
                    }
                    break;

                case sx_tool_act.sx_tool_apply:
                    RaisePropertyChangedEventAsync("IsSaveAvailable");
                    break;
            }
        }

    }
}